@@ -129,6 +129,33 @@ class Animations:  # {{{
 # }}}
 
 
+class OverridableConfig:
+    "A `dict`-like object allowing per-monitor overrides"
+
+    def __init__(self, ref, monitor_override):
+        self.ref = ref
+        self.mon_override = monitor_override
+
+    def __setitem__(self, name, value):
+        self.ref[name] = value
+
+    def __getitem__(self, name):
+        override = self.mon_override.get(state.active_monitor, {})
+        if name in override:
+            return override[name]
+        return self.ref[name]
+
+    def get(self, name, default=None):
+        "get the attribute `name`"
+        try:
+            return self[name]
+        except KeyError:
+            return default
+
+    def __str__(self):
+        return f"{self.ref} {self.mon_override}"
+
+
 class Scratch(CastBoolMixin):  # {{{
     "A scratchpad state including configuration & client state"
     log = logging.getLogger("scratch")
@@ -137,10 +164,12 @@ class Scratch(CastBoolMixin):  # {{{
     def __init__(self, uid, opts):
         self.uid = uid
         self.pid = 0
-        self.conf = opts
+        if self.cast_bool(opts.get("preserve_aspect")):
+            opts["lazy"] = True
         if not opts.get("process_tracking", True):
-            self.conf["lazy"] = True
-            self.conf["class_match"] = True
+            opts["lazy"] = True
+            opts["class_match"] = True
+        self.conf = OverridableConfig(opts, opts.get("monitor", {}))
         self.visible = False
         self.client_info = {}
         self.should_hide = False
@@ -149,6 +178,15 @@ class Scratch(CastBoolMixin):  # {{{
         self.space_identifier = None
         self.monitor = ""
 
+    async def get_auto_offset(self, monitor=None):
+        "Get au automatic offset value computed from client size"
+        if monitor is None:
+            monitor = await get_focused_monitor_props(self.log)
+        return map(
+            int,
+            [monitor["width"] / monitor["scale"], monitor["height"] / monitor["scale"]],
+        )
+
     async def initialize(self, ex):
         "Initialize the scratchpad"
         if self.initialized:
@@ -363,6 +401,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def on_reload(self) -> None:
         "config loader"
+        # Create new scratches with fresh config items
         scratches = {
             name: Scratch(name, options) for name, options in self.config.items()
         }
@@ -371,8 +410,10 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         for name in scratches:
             scratch = self.scratches.get(name)
             if scratch:
+                # if existing scratch exists, overrides the conf object
                 scratch.conf = scratches[name].conf
             else:
+                # else register it
                 self.scratches.register(scratches[name], name)
                 is_lazy = self.cast_bool(scratches[name].conf.get("lazy"), False)
                 if not is_lazy:
@@ -414,15 +455,24 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                 "fromleft": f"-200% {margin_y}",
             }[animation_type]
 
-            await self.hyprctl(
-                [
-                    f"windowrule workspace special:scratch_{scratch.uid} silent,^({defined_class})$",
-                    f"windowrule float,^({defined_class})$",
-                    f"windowrule move {t_pos},^({defined_class})$",
-                    f"windowrule size {width} {height},^({defined_class})$",
-                ],
-                "keyword",
-            )
+            ipc_commands = [
+                f"windowrule float,^({defined_class})$",
+            ]
+
+            if not self.cast_bool(scratch.conf.get("lazy")):
+                ipc_commands.append(
+                    f"windowrule workspace special:scratch_{scratch.uid} silent,^({defined_class})$"
+                )
+
+            if not self.cast_bool(scratch.conf.get("preserve_aspect")):
+                ipc_commands.extend(
+                    [
+                        f"windowrule move {t_pos},^({defined_class})$",
+                        f"windowrule size {width} {height},^({defined_class})$",
+                    ]
+                )
+
+            await self.hyprctl(ipc_commands, "keyword")
 
     async def __wait_for_client(self, item, use_proc=True) -> bool:
         """Waits for a client to be up and running
@@ -659,11 +709,11 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
     async def run_toggle(self, uid_or_uids: str) -> None:
         """<name> toggles visibility of scratchpad "name" """
         if " " in uid_or_uids:
-            uids = [x.strip() for x in uid_or_uids.split() if x.strip()]
+            uids = list(filter(bool, map(str.strip, uid_or_uids.split())))
         else:
             uids = [uid_or_uids.strip()]
 
-        assert len(uids)
+        assert len(uids) > 0
         first_scratch = self.scratches.get(uids[0])
         if not first_scratch:
             self.log.warning("%s doesn't exist, can't toggle.", uids[0])
@@ -698,17 +748,20 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             if "size" not in scratch.client_info:
                 await self.updateScratchInfo(scratch)
 
-            offset = int(1.3 * scratch.client_info["size"][1])
+            off_x, off_y = await scratch.get_auto_offset()
+        await self._slide_animation(animation_type, scratch, off_x, off_y)
 
+    async def _slide_animation(self, animation_type, scratch, off_x, off_y):
+        "Slides the window `offset` pixels respecting `animation_type`"
         addr = "address:" + scratch.full_address
         if animation_type == "fromtop":
-            await self.hyprctl(f"movewindowpixel 0 -{offset},{addr}")
+            await self.hyprctl(f"movewindowpixel 0 -{off_y},{addr}")
         elif animation_type == "frombottom":
-            await self.hyprctl(f"movewindowpixel 0 {offset},{addr}")
+            await self.hyprctl(f"movewindowpixel 0 {off_y},{addr}")
         elif animation_type == "fromleft":
-            await self.hyprctl(f"movewindowpixel -{offset} 0,{addr}")
+            await self.hyprctl(f"movewindowpixel -{off_x} 0,{addr}")
         elif animation_type == "fromright":
-            await self.hyprctl(f"movewindowpixel {offset} 0,{addr}")
+            await self.hyprctl(f"movewindowpixel {off_x} 0,{addr}")
 
         if self.scratches.hasState(scratch, "transition"):
             return  # abort sequence
@@ -732,6 +785,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             return
 
         self.log.info("Showing %s", uid)
+        was_alive = await item.isAlive()
         if not await self.ensure_alive(uid):
             self.log.error("Failed to show %s, aborting.", uid)
             return
@@ -755,30 +809,50 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         assert monitor
         assert item.full_address, "No address !"
 
-        animation_type = item.conf.get("animation", "").lower()
-        wrkspc = monitor["activeWorkspace"]["id"]
-
         self.scratches.setState(item, "transition")
         item.monitor = monitor["name"]
+        await self._show_transition(item, monitor, was_alive)
+
+    async def _show_transition(self, item, monitor, was_alive):
+        "perfoms the transition to visible state"
+        animation_type = item.conf.get("animation", "").lower()
+        wrkspc = monitor["activeWorkspace"]["id"]
         # Start the transition
         await self.hyprctl(
             [
-                f"moveworkspacetomonitor special:scratch_{uid} {item.monitor}",
+                f"moveworkspacetomonitor special:scratch_{item.uid} {item.monitor}",
                 f"movetoworkspacesilent {wrkspc},address:{item.full_address}",
                 f"alterzorder top,address:{item.full_address}",
             ]
         )
-        await self._fix_size(item, monitor)
+        preserve_aspect = self.cast_bool(item.conf.get("preserve_aspect"))
+        should_set_aspect = not (
+            preserve_aspect and was_alive
+        )  # not aspect preserving or it's newly spawned
+        if should_set_aspect:
+            await self._fix_size(item, monitor)
         await item.updateClientInfo()
-        if not await self._fix_position(item, monitor):
-            # position wasn't provided, use animation + margin
+        position_fixed = False
+        if should_set_aspect:
+            position_fixed = await self._fix_position(item, monitor)
+        if not position_fixed:
             if animation_type:
-                margin = item.conf.get("margin", DEFAULT_MARGIN)
-                fn = getattr(Animations, animation_type)
-                command = fn(
-                    monitor, item.client_info, "address:" + item.full_address, margin
-                )
-                await self.hyprctl(command)
+                if preserve_aspect and was_alive:
+                    if "size" not in item.client_info:
+                        await self.updateScratchInfo(item)
+
+                    ox, oy = await item.get_auto_offset(monitor)
+                    await self._slide_animation(animation_type, item, -ox, -oy)
+                else:
+                    margin = item.conf.get("margin", DEFAULT_MARGIN)
+                    fn = getattr(Animations, animation_type)
+                    command = fn(
+                        monitor,
+                        item.client_info,
+                        "address:" + item.full_address,
+                        margin,
+                    )
+                    await self.hyprctl(command)
             else:
                 await self.notify_error(
                     f"No position and no animation provided for {item.uid}, don't know where to place it."
@@ -792,6 +866,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def _fix_size(self, item, monitor):
         "apply the `size` config parameter"
+
         size = item.conf.get("size")
         if size:
             width, height = convert_coords(self.log, size, monitor)
